from string import ascii_lowercase


class BaseName(str):
    """
    Base validation of kubernetes names. All other types may inherit for this one
    to get base validation.
    """
    NUMBERS = "0123456789"
    LEGAL_CHARS = ascii_lowercase + "-" + NUMBERS

    @classmethod
    def __get_validators__(cls):
        yield cls.base_name_validate

    @classmethod
    def base_name_validate(cls, name: str):
        cls.is_string(name)
        cls.is_empty_string(name)
        cls.has_upper_case(name)
        cls.starts_with_number(name)
        cls.starts_with_dash(name)

        return name

    @staticmethod
    def is_string(name):
        if not isinstance(name, str):
            raise TypeError('string required')
        return name

    @staticmethod
    def is_empty_string(name):
        if name == "":
            raise ValueError("String cannot be empty")
        return name

    @classmethod
    def has_upper_case(cls, name: str):
        if not all(char in cls.LEGAL_CHARS for char in name):
            raise ValueError(f'Value maybe only container lowe case ascii characters, and dash: "{name}"')
        return name

    @classmethod
    def starts_with_number(cls, name):
        if name[0] in cls.NUMBERS:
            raise ValueError(f'Value cannot start with a number: "{name}"')
        return name

    @staticmethod
    def starts_with_dash(name):
        if name[0] == "-":
            raise ValueError(f'Value cannot start with a dash: "{name}"')
        return name



from pydantic_custom_types.base_name import BaseName


class NamespaceName(BaseName):
    @classmethod
    def __get_validators__(cls):
        yield cls.validate_namespace_name

    @classmethod
    def validate_namespace_name(cls, name: str) -> str:
        cls.base_name_validate(name)
        return name

    @classmethod
    def has_to_many_characters(cls, name: str):
        max_length = 64
        if len(name) >= max_length:
            raise ValueError(f"Name cannot be longer than {max_length} characters")

        return name



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pydantic_custom_types', 'pydantic_custom_types.kubernetes']

package_data = \
{'': ['*']}

setup_kwargs = {
    'name': 'pydantic-custom-types',
    'version': '0.0.1',
    'description': 'Custom types for pydantic used in SRE peipelins for validation input vars',
    'long_description': '# pydantic custom types\ndistributes custom types instead of copy pasting code for pipelines input validation',
    'author': 'per lejon',
    'author_email': 'per.lejon@netigate.se',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
